@@ -1,6 +1,9 @@
 agent_mode=False
 prompt_engineering=False
 max_steps=30
+lite=True
 hide_tools_order=True
 default_bible="NET"
-max_semantic_matches=15
\ No newline at end of file
+max_semantic_matches=15
+max_log_lines=2000
+disabled_tools=[]
\ No newline at end of file
