@@ -2,14 +2,15 @@ from biblemate.core.systems import *
 from biblemate.ui.prompts import getInput
 from biblemate.ui.info import get_banner
 from biblemate.ui.selection_dialog import TerminalModeDialogs
-from biblemate import config, AGENTMAKE_CONFIG, OLLAMA_NOT_FOUND
+from biblemate import config, AGENTMAKE_CONFIG, OLLAMA_NOT_FOUND, fix_string
 from biblemate.core.bible_db import BibleVectorDatabase
 from pathlib import Path
-import asyncio, re, os, subprocess, click, shutil, pprint
+import asyncio, re, os, subprocess, click, shutil, pprint, argparse
 from copy import deepcopy
 from alive_progress import alive_bar
 from fastmcp import Client
 from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_configurations, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_AI_BACKEND
+from agentmake.utils.handle_text import set_log_file_max_lines
 from rich.console import Console
 from rich.markdown import Markdown
 from rich.progress import Progress, SpinnerColumn, TextColumn
@@ -17,14 +18,22 @@ from rich.terminal_theme import MONOKAI
 if not USER_OS == "Windows":
     import readline  # for better input experience
 
+# trim long log file
+log_path = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "logs")
+if not os.path.isdir(log_path):
+    Path(log_path).mkdir(parents=True, exist_ok=True)
+log_file = os.path.join(log_path, "requests")
+set_log_file_max_lines(log_file, config.max_log_lines)
+
 # bible data
 builtin_bible_data = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data", "bibles")
 user_bible_data = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "data", "bibles")
-Path(user_bible_data).mkdir(parents=True, exist_ok=True)
-user_bible = os.path.join(user_bible_data, "NET.bible")
+if not os.path.isdir(user_bible_data):
+    Path(user_bible_data).mkdir(parents=True, exist_ok=True)
+user_bible = os.path.join(user_bible_data, f"{config.default_bible}.bible")
 if not os.path.isfile(user_bible):
     print("# Copying bible data ...")
-    shutil.copyfile(os.path.join(builtin_bible_data, "NET.bible"), user_bible)
+    shutil.copyfile(os.path.join(builtin_bible_data, f"{config.default_bible}.bible"), user_bible)
 if os.path.isfile(user_bible) and os.path.getsize(user_bible) < 380000000:
     if shutil.which("ollama"):
         print("# Setting up a bible vector database to support semantic search with BibleMate AI. please kindly wait until it is finished ...")
@@ -35,6 +44,21 @@ if os.path.isfile(user_bible) and os.path.getsize(user_bible) < 380000000:
     else:
         print(OLLAMA_NOT_FOUND)
 
+# AI backend
+parser = argparse.ArgumentParser(description = """BibleMate AI CLI options""")
+parser.add_argument("-b", "--backend", action="store", dest="backend", help="AI backend; overrides the default backend temporarily.")
+args = parser.parse_args()
+# write to the `config.py` file temporarily for the MCP server to pick it up
+if args.backend:
+    with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.py"), "a", encoding="utf-8") as fileObj:
+        fileObj.write(f'''\nbackend="{args.backend}"''')
+    config.backend = args.backend
+else:
+    with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.py"), "a", encoding="utf-8") as fileObj:
+        fileObj.write(f'''\nbackend="{DEFAULT_AI_BACKEND}"''')
+    config.backend = DEFAULT_AI_BACKEND
+AGENTMAKE_CONFIG["backend"] = config.backend
+
 # The client that interacts with the Bible Study MCP server
 builtin_mcp_server = os.path.join(os.path.dirname(os.path.realpath(__file__)), "bible_study_mcp.py")
 user_mcp_server = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "bible_study_mcp.py") # The user path has the same basename as the built-in one; users may copy the built-in server settings to this location for customization. 
@@ -67,11 +91,12 @@ async def initialize_app(client):
     if "get_direct_text_response" not in available_tools:
         available_tools.insert(0, "get_direct_text_response")
     master_available_tools = deepcopy(available_tools)
+    available_tools = [i for i in available_tools if not i in config.disabled_tools]
 
     tool_descriptions = ""
     if "get_direct_text_response" not in tools:
         tool_descriptions = """# TOOL DESCRIPTION: `get_direct_text_response`
-Get a static text-based response directly from a text-based AI model without using any other tools. This is useful when you want to provide a simple and direct answer to a question or request, without the need for online latest updates or task execution.\n\n\n"""
+Get a static text-based response directly from a text-based AI model without using any other tools. This is useful when you want to provide a simple and direct answer to a question or request, without the need for online latest updates or task execution."""
     for tool_name, tool_description in tools.items():
         tool_descriptions += f"""# TOOL DESCRIPTION: `{tool_name}`
 {tool_description}\n\n\n"""
@@ -107,8 +132,9 @@ Get a static text-based response directly from a text-based AI model without usi
 def backup_conversation(console, messages, master_plan):
     """Backs up the current conversation to the user's directory."""
     timestamp = getCurrentDateTime()
-    storagePath = os.path.join(AGENTMAKE_USER_DIR, "biblemate", timestamp)
-    Path(storagePath).mkdir(parents=True, exist_ok=True)
+    storagePath = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "chats", timestamp)
+    if not os.path.isdir(storagePath):
+        Path(storagePath).mkdir(parents=True, exist_ok=True)
     # Save full conversation
     conversation_file = os.path.join(storagePath, "conversation.py")
     writeTextFile(conversation_file, pprint.pformat(messages))
@@ -116,7 +142,7 @@ def backup_conversation(console, messages, master_plan):
     writeTextFile(os.path.join(storagePath, "master_plan.md"), master_plan)
     # Save markdown
     markdown_file = os.path.join(storagePath, "conversation.md")
-    markdown_text = "\n\n".join([f"```{i["role"]}\n{i["content"]}\n```" for i in messages if i.get("role", "") in ("user", "assistant")])
+    markdown_text = "\n\n".join(["```"+i["role"]+"\n"+i["content"]+"\n```" for i in messages if i.get("role", "") in ("user", "assistant")])
     writeTextFile(markdown_file, markdown_text)
     # Save html
     html_file = os.path.join(storagePath, "conversation.html")
@@ -131,9 +157,12 @@ def write_user_config():
     configurations = f"""agent_mode={config.agent_mode}
 prompt_engineering={config.prompt_engineering}
 max_steps={config.max_steps}
+lite={config.lite}
 hide_tools_order={config.hide_tools_order}
 default_bible="{config.default_bible}"
-max_semantic_matches={config.max_semantic_matches}"""
+max_semantic_matches={config.max_semantic_matches}
+max_log_lines={config.max_log_lines}
+disabled_tools={pprint.pformat(config.disabled_tools)}"""
     writeTextFile(config_file, configurations)
 
 async def main_async():
@@ -149,6 +178,7 @@ async def main_async():
 
     async with client:
         tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema = await initialize_app(client)
+        write_user_config() # remove the temporary `config.backend`
         
         available_tools_pattern = "|".join(available_tools)
         prompt_list = [f"/{p}" for p in prompts.keys()]
@@ -296,6 +326,8 @@ async def main_async():
                     )
                     if enabled_tools is not None:
                         available_tools = enabled_tools
+                        config.disabled_tools = [i for i in master_available_tools if not i in available_tools]
+                        write_user_config()
                     console.rule()
                     tools_descriptions = [f"- `{name}`: {description}" for name, description in tools.items()]
                     console.print(Markdown("## Available Tools\n\n"+"\n".join(tools_descriptions)))
@@ -368,6 +400,38 @@ async def main_async():
             # Check if a single tool is specified
             specified_prompt = ""
             specified_tool = ""
+
+            # Tool selection systemm message
+            system_tool_selection = get_system_tool_selection(available_tools, tool_descriptions)
+
+            if user_request.startswith("@ "):
+                user_request = user_request[2:].strip()
+                # Single Tool Suggestion
+                suggested_tools = []
+                async def get_tool_suggestion():
+                    nonlocal suggested_tools, user_request, system_tool_selection
+                    if DEVELOPER_MODE and not config.hide_tools_order:
+                        console.print(Markdown(f"## Tool Selection (descending order by relevance)"), "\n")
+                    else:
+                        console.print(Markdown(f"## Tool Selection"), "\n")
+                    # Extract suggested tools from the step suggestion
+                    suggested_tools = agentmake(user_request, system=system_tool_selection, **AGENTMAKE_CONFIG)[-1].get("content", "").strip() # Note: suggested tools are printed on terminal by default, could be hidden by setting `print_on_terminal` to false
+                    suggested_tools = re.sub(r"^.*?(\[.*?\]).*?$", r"\1", suggested_tools, flags=re.DOTALL)
+                    try:
+                        suggested_tools = eval(suggested_tools.replace("`", "'")) if suggested_tools.startswith("[") and suggested_tools.endswith("]") else ["get_direct_text_response"] # fallback to direct response
+                    except:
+                        suggested_tools = ["get_direct_text_response"]
+                await thinking(get_tool_suggestion)
+                # Single Tool Selection
+                if config.agent_mode:
+                    this_tool = suggested_tools[0] if suggested_tools else "get_direct_text_response"
+                else: # `partner` mode when config.agent_mode is set to False
+                    this_tool = await dialogs.getValidOptions(options=suggested_tools if suggested_tools else available_tools, title="Suggested Tools", text="Select a tool:")
+                    if not this_tool:
+                        this_tool = "get_direct_text_response"
+                # Re-format user request
+                user_request = f"@{this_tool} " + user_request
+
             if re.search(prompt_pattern, user_request):
                 specified_prompt = re.search(prompt_pattern, user_request).group(1)
                 user_request = user_request[len(specified_prompt):]
@@ -410,6 +474,8 @@ async def main_async():
 
             async def run_tool(tool, tool_instruction):
                 nonlocal messages
+                tool_instruction = fix_string(tool_instruction)
+                messages[-1]["content"] = fix_string(messages[-1]["content"])
                 if tool == "get_direct_text_response":
                     messages = agentmake(messages, system="auto", **AGENTMAKE_CONFIG)
                 else:
@@ -417,9 +483,14 @@ async def main_async():
                         tool_schema = tools_schema[tool]
                         tool_properties = tool_schema["parameters"]["properties"]
                         if len(tool_properties) == 1 and "request" in tool_properties: # AgentMake MCP Servers or alike
-                            tool_result = await client.call_tool(tool, {"request": tool_instruction})
+                            if "items" in tool_properties["request"]: # requires a dictionary instead of a string
+                                request_dict = [{"role": "system", "content": DEFAULT_SYSTEM}]+messages[len(messages)-2:] if config.lite else messages
+                                request_dict += [{"role": "user", "content": tool_instruction}]
+                                tool_result = await client.call_tool(tool, {"request": request_dict})
+                            else:
+                                tool_result = await client.call_tool(tool, {"request": tool_instruction})
                         else:
-                            structured_output = getDictionaryOutput(messages=messages, schema=tool_schema, backend=DEFAULT_AI_BACKEND)
+                            structured_output = getDictionaryOutput(messages=messages, schema=tool_schema, backend=config.backend)
                             tool_result = await client.call_tool(tool, structured_output)
                         tool_result = tool_result.content[0].text
                         messages[-1]["content"] += f"\n\n[Using tool `{tool}`]"
@@ -428,6 +499,7 @@ async def main_async():
                         if DEVELOPER_MODE:
                             console.print(f"Error: {e}\nFallback to direct response...\n\n")
                         messages = agentmake(messages, system="auto", **AGENTMAKE_CONFIG)
+                messages[-1]["content"] = fix_string(messages[-1]["content"])
 
             # user specify a single tool
             if specified_tool and not specified_tool == "@@":
@@ -455,7 +527,7 @@ async def main_async():
                     if len(prompt_properties) == 1 and "request" in prompt_properties: # AgentMake MCP Servers or alike
                         result = await client.get_prompt(specified_prompt[1:], {"request": user_request})
                     else:
-                        structured_output = getDictionaryOutput(messages=messages, schema=prompt_schema, backend=DEFAULT_AI_BACKEND)
+                        structured_output = getDictionaryOutput(messages=messages, schema=prompt_schema, backend=config.backend)
                         result = await client.get_prompt(specified_prompt[1:], structured_output)
                     #print(result, "\n\n")
                     master_plan = result.messages[0].content.text
@@ -506,11 +578,8 @@ Available tools are: {available_tools}.
                     console.print(Markdown(master_plan), "\n\n")
 
             # Step suggestion system message
-            system_progress = get_system_progress(original_request=user_request, master_plan=master_plan)
-            system_make_suggestion = get_system_make_suggestion(original_request=user_request, master_plan=master_plan)
-
-            # Tool selection systemm message
-            system_tool_selection = get_system_tool_selection(available_tools, tool_descriptions)
+            system_progress = get_system_progress(master_plan=master_plan)
+            system_make_suggestion = get_system_make_suggestion(master_plan=master_plan)
 
             # Get the first suggestion
             next_suggestion = "START"
@@ -550,6 +619,8 @@ Available tools are: {available_tools}.
                     next_tool = suggested_tools[0] if suggested_tools else "get_direct_text_response"
                 else: # `partner` mode when config.agent_mode is set to False
                     next_tool = await dialogs.getValidOptions(options=suggested_tools if suggested_tools else available_tools, title="Suggested Tools", text="Select a tool:")
+                    if not next_tool:
+                        next_tool = "get_direct_text_response"
                 prefix = f"## Next Tool [{step}]\n\n" if DEVELOPER_MODE and not config.hide_tools_order else ""
                 console.print(Markdown(f"{prefix}`{next_tool}`"))
                 print()
@@ -617,7 +688,9 @@ Please provide me with the final answer to my original request based on the work
 
 # Original Request
 {user_request}""",
-                stream=True)
+                stream=True,
+            )
+            messages[-1]["content"] = fix_string(messages[-1]["content"])
             console.rule()
             console.print(Markdown(messages[-1]['content']))
 
